/* 8.2 AMS 17jan2008 */

program dca

	version 8.2

	syntax varlist(min=2 numeric), [legendoff graphoff xstart(real 0.01) xstop(real 0.99) ///
						  xby(real 0.01) ymin(real -0.05) ymax(real 1.0)       ///
						  clpat(string) clcolor(string) prob(string)          ///
						  saving(string) replace harm(numlist) decimal(integer 2) ///
						  smooth smoother(string) INTERvention intermin(real 0) ]

	/* make it so that, after the program is run, the data set remains unchanged */
	preserve

	/* parse the syntax into fail and predictor variables */
	gettoken fail 0: 0, parse(" ")
	global fail = "`fail'"
	gettoken predictors 0: 0, parse(",")
	local predictors = "`predictors'"


	/* drop patients with missing values for fail variable */
	quietly count if $fail == .
	if r(N) > 0 {
		di as result "note: " r(N) " records dropped due to missing data"
		quietly drop if $fail==.
		}


	/* check for errors in variables passed to the program */
	quietly sum $fail
	quietly count if $fail ~=r(min) & $fail~=r(max)
	if r(N)>0 {
		quietly sum $fail
		if r(min)>=0 & r(max)<=1 {
			disp as result "note: decision curve is computed by the predicted probabilities 
			disp as result "      and without reference to actual outcome"
			}
		else {
			disp as error "error: failure variable must be binary and coded as 0 for non-failure and 1 for failure"
			exit 198
			}
		}
	if `xstart' < 0 | `xstart' > 1 {
		di as error "error: xstart must lie between 0 and 1"
		exit 198
		}
	if `xstop' < 0 | `xstop' > 1 {
		di as error "error: xstop must lie between 0 and 1"
		exit 198
		}
	if `xby' < 0 | `xby' > 1 {
		di as error "error: xby must lie between 0 and 1"
		exit 198
		}
	if `xstop' <= `xstart' {
		di as error "error: xstop must be larger than xstart"
		exit 198
		}

	/* check for errors in the decimal option */
	if `decimal' < 0 {
		di as error "error: decimal cannot take on a negative value"
		exit 198
		}


	/* check for errors in the prob option */
	if "`prob'" ~="" {
		if wordcount("`prob'") ~= wordcount("`predictors'") {
			di as error "error: prob option misspecified"
			exit 198
			}
		local index = 0
		foreach pr of local prob {
			local index = `index' + 1
			local prob`index' = upper("`pr'")
			}
		}

	else {
		local index = 0
		foreach p of local predictors {
			local index = `index' + 1
			local prob`index' = "YES"
			}
		}
		
	local index = 0
	foreach p of local predictors {
		local index = `index' + 1
		if "`prob`index''" == "YES" {
			quietly sum `p'
			if r(max) > 1 | r(min) < 0 {
				di as error "error: prob option misspecified"
				exit 198
				}
			}
		}

	/* set defaults for options, if not specified by user */
	if "`graphoff'" ~="" {
		local graph = "no"
		local legend = "off"
		}
	else {
		if "`legendoff'" ~="" {
			local graph = "yes"
			local legend = "off"
			}
		else {
			local graph = "yes"
			local legend = "on"
			}
		if "`clpat'" == "" {
			di as result "note: no line pattern specified: default line patterns used"
			local clpat = "dash solid dot"
			}
		if "`clcolor'" == "" {
			di as result "note: no line color specified: default line colors used"
			local clcolor = "gs0 gs0 gs0"
			}
		}


	/* tokenize the harm numlist */
	/* the harm associated with each predictor is saved in local p1harm, p2harm, etc. */
	tokenize `harm'
	local indexharm = 0
	if "`harm'" == "" {
		di as result "note: no harm associated with any xvars"
		foreach p of local predictors {
			local indexharm = `indexharm' + 1
			local p`indexharm'harm = 0
			}
		}

	else {
		if wordcount("`predictors'") ~= wordcount("`harm'") {
			di as error "error: harm misspecified"
			exit 198
			}
		else {
			foreach p of local predictors {
				local indexharm = `indexharm' + 1
				local p`indexharm'harm = ``indexharm''
				}
			}
		}

	/* generate message and set default when smooth is not specified */
	if "`smooth'" == "" {
		di as result "note: no smoothing applied to curves"
		}
	else {
		if "`smoother'" == "" {
			di as result "note: no smoother specified; default smoothing applied"
			local smooth = "3rssh" 
			}
		else {
			local smooth = "`smother'" 
			}
		}


	/* here is the start of calculations */		
	******************************************************************************************************************
	*** set observations to accommodate all thresholds analyzed
	******************************************************************************************************************
	local obsrequired = round((`xstop' - `xstart') / `xby')
	quietly count
	if r(N) < `obsrequired' {
		quietly set obs `obsrequired'
		}

	******************************************************************************************************************
	*** initialize variables 
	******************************************************************************************************************
	tempvar threshold all none
	quietly g `threshold' = .
	quietly g `all' = .
	quietly g `none' = 0
	label var `threshold' "Threshold probability"
	label var `all' "all"
	label var `none' "none"
	
	
	******************************************************************************************************************
	*** fit models, save out predicted probabilities, and set up variables which will contain net benefits
	******************************************************************************************************************
	local index = 0
	local models = ""
	global modellist = ""

	foreach p of local predictors {
		local index = `index' + 1
		if "`prob`index''" == "YES" {
			g p`index' = `p'
			}
		else {
			quietly logistic $fail `p', nolog
			quietly predict p`index'
			}
		local models = "`models'" + " p`index'"
		quietly g modelp`index' = .
		label var modelp`index' "`p'"
		global modellist = "$modellist" + "  " + "modelp`index'"
		}

	global numberofmodels=wordcount("$modellist")
	
	** set sample = 1 if data are non-missing for all models
	** (the goal is to compare the benefits for the same sample of patients)
	local temp = "p1 < ."
	if `index' > 1 {
		forvalues i=1(1)`index' {
			local temp = "`temp'" + " & p`i' < . "
			}
		}
	g sample = `temp'
	
	
	******************************************************************************************************************
	*** calculate net benefit for each threshold
	******************************************************************************************************************
	local l = 1
	forvalues j=`xstart'(`xby')`xstop' {
		quietly replace `threshold'=`j' in `l'
		foreach m of local models {
			quietly sum $fail if `m' >= `j' & sample==1
			local a = r(N) * r(mean)
			local b = r(N) * (1 - r(mean))
			quietly count if sample==1
			local n = r(N)
			local u=(`a'-`b'*(`j'/(1-`j')))/`n'
			quietly replace model`m' =`u' - ``m'harm' in `l'
			}
		quietly sum $fail if sample==1
		local allnb = r(mean) - (1 - r(mean)) * `j' / (1 - `j')
		quietly replace `all' = `allnb' in `l'

		local l=`l'+1
		}
	 
	*put threshold on a percentage scale
	quietly replace `threshold'=`threshold'*100
	
	* transform net benefit to interventions, if specified
	if "`intervention'" ~="" {
		foreach var of varlist $modellist {
			quietly replace `var'= (`var' - `all') * 100 / ( `threshold' / (100 - `threshold') )
			if "`intermin'" ~="" {
				quietly replace `var' = . if `var' < `intermin'
				}
			}	
		}


	else {
		* cut off the curves if they are too low
		foreach var of varlist $modellist `all' {
			quietly replace `var'=. if `var'< `ymin' | `var' > `ymax' 
			}
		}

	
	******************************************************************************************************************
	*** save output from decision curve analysis
	******************************************************************************************************************
	keep $modellist `all' `none' `threshold'
	quietly drop if `threshold'==.
	* the temp variables currently have anonomous names
	* rename them here
	rename __000000 threshold
	rename __000001 all
	rename  __000002 none
	if "`saving'" ~="" {
		save "`saving'", `replace'
		}

	******************************************************************************************************************
	*** apply smoothing, if specified
	******************************************************************************************************************
	if "`smooth'" ~= "" {
		foreach var of varlist $modellist {
			quietly smooth `smooth' `var', gen(sm`var')
			quietly drop `var'
			quietly rename sm`var' `var'
			}
		}
	
	******************************************************************************************************************
	*** graph
	******************************************************************************************************************
	if "`graph'"=="yes"{

		if "`intervention'" == "" {

			if `ymin' > 0 {
				twoway (line all $modellist threshold if threshold>=`xstart'*100 & threshold<=`xstop'*100, sort  /*
				*/ clpat(solid `clpat') clwidth(medthick medium medium medium) clcolor(gs0 gs10 `clcolor')), /*
				*/  scheme(s1mono) xtitle(Threshold probability in %, margin(medium)) ytitle(Net Benefit, margin(medium)) /*
				*/  ylabel(, format(%9.`decimal'f)) legend(`legend')
				}
		
			else {
				twoway (line none all $modellist threshold if threshold>=`xstart'*100 & threshold<=`xstop'*100, sort  /*
				*/ clpat(solid solid `clpat') clwidth(medthick medthick medium medium medium) clcolor(gs0 gs10 `clcolor')), /*
				*/  scheme(s1mono) xtitle(Threshold probability in %, margin(medium)) ytitle(Net Benefit, margin(medium)) /*
				*/  ylabel(, format(%9.`decimal'f)) legend(`legend')
				}
		
			*************************************************************************************************************
			*** print out a line of text for a figure title
			*************************************************************************************************************
			
			if "`legend'" == "off" {
				disp ""
				disp "Figure legend: "
				if `ymin' > 0 {
					disp "Solid black line is treat none"
					}
				disp "Solid grey line is treat all"
				forvalues i=1(1)$numberofmodels{
					disp proper(word("`clpat'",`i')), word("`clcolor'",`i') " line is treat based on model:",  word("`predictors'",`i')
					}
				}
			}

		else {
			twoway (line $modellist threshold if threshold>=`xstart'*100 & threshold<=`xstop'*100, sort  /*
			*/ clpat(`clpat') clwidth(medium medium medium) clcolor(`clcolor')), /*
			*/  scheme(s1mono) xtitle(Threshold probability in %, margin(medium)) ytitle("Net reduction in interventions" "per 100 patients", margin(medium)) /*
			*/  ylabel(, format(%9.0f)) legend(`legend')
		
		
			*************************************************************************************************************
			*** print out a line of text for a figure title
			*************************************************************************************************************
			
			if "`legend'" == "off" {
				disp ""
				disp "Figure legend: "
				forvalues i=1(1)$numberofmodels{
					disp proper(word("`clpat'",`i')), word("`clcolor'",`i') " line is treat based on model:",  word("`predictors'",`i')
					}
				}
			}


		}
	restore

end
exit
