/* version 8.2 */
/* AMC created 13april2007 */
/* AMC last modified 04apr2008 */

program stdca

	version 9.0

	syntax varlist(min=1 numeric), timepoint(real) [legendoff graphoff xstart(real 0.01)	///
					       xstop(real 0.99) xby(real 0.01) ymin(real -0.05) 	///
						 ymax(real 1.0) clpat(string) clcolor(string)		///
						  saving(string) replace harm(numlist) decimal(integer 2)	///
						  smooth smoother(string) INTERvention intermin(real 0) cmprsk(varname)]


	di as text " "
	di as text "Calculating decision curve. This may take a few moments..."
	di as text " "

	/* make it so that, after the program is run, the data set remains unchanged */
	preserve

	/* check that the data is stset */
	/* if not, then the program will automatically generate an error */
	quietly sts list, at(0.1 0.2)


	/* parse the syntax into time, failure, and predictor variables */
	gettoken predictors 0: 0, parse(",")
	local predictors = "`predictors'"


	/* check for errors in variables passed to the program */
	if `timepoint' <= 0 {
		di as error "error: timepoint must be greater than 0"
		exit 198
		}

	if `xstart' < 0 | `xstart' > 1 {
		di as error "error: xstart must lie between 0 and 1"
		exit 198
		}
	if `xstop' < 0 | `xstop' > 1 {
		di as error "error: xby must lie between 0 and 1"
		exit 198
		}
	if `xby' < 0 | `xby' > 1 {
		di as error "error: xby must lie between 0 and 1"
		exit 198
		}
	if `xstop' <= `xstart' {
		di as error "error: xstop must be larger than xstart"
		exit 198
		}

	/* check for errors in the decimal option */
	if `decimal' < 0 {
		di as error "error: decimal cannot take on a negative value"
		exit 198
		}


	/* set defaults for options, if not specified by user */
	if "`graphoff'" ~="" {
		local graph = "no"
		local legend = "off"
		}
	else {
		if "`legendoff'" ~="" {
			local graph = "yes"
			local legend = "off"
			}
		else {
			local graph = "yes"
			local legend = "on"
			}
		if "`clpat'" == "" {
			di as result "note: no line pattern specified: default line patterns used"
			local clpat = "dash solid dot"
			}
		if "`clcolor'" == "" {
			di as result "note: no line color specified: default line colors used"
			local clcolor = "gs0 gs0 gs0"
			}
		}

	/* tokenize the harm numlist */
	/* the harm associated with each predictor is saved in local p1harm, p2harm, etc. */
	tokenize `harm'
	local indexharm = 0
	if "`harm'" == "" {
		di as result "note: no harm associated with any xvars"
		foreach p of local predictors {
			local indexharm = `indexharm' + 1
			local p`indexharm'harm = 0
			}
		}

	else {
		if wordcount("`predictors'") ~= wordcount("`harm'") {
			di as error "error: harm misspecified"
			exit 198
			}
		else {
			foreach p of local predictors {
				local indexharm = `indexharm' + 1
				local p`indexharm'harm = ``indexharm''
				}
			}
		}

	/* generate message and set default when smooth is not specified */
	if "`smooth'" == "" {
		di as result "note: no smoothing applied to curves"
		}
	else {
		if "`smoother'" == "" {
			di as result "note: no smoother specified; default smoothing applied"
			local smooth = "3rssh" 
			}
		else {
			local smooth = "`smother'" 
			}
		}

	/* check that all predictors are probabilities */
	local index = 0
	foreach p of local predictors {
		local index = `index' + 1
		quietly sum `p'
		if r(max) > 1 | r(min) < 0 {
			di as error "error: xvars must be probabilities and lie between 0 and 1"
			exit 198
			}
		}

	/* check that all predictors represent the probability of the event */
	local index = 0
	foreach p of local predictors {
		local index = `index' + 1
		quietly cor `p' _d
		if r(rho) < 0 {
			quietly replace `p' = 1 - `p'
			di as error "note: `p' was subtracted from unity so that it represents the probability of failure"
			}
		}



	/* here is the start of calculations */		
	******************************************************************************************************************
	*** set observations to accommodate all thresholds analyzed
	******************************************************************************************************************
	local obsrequired = round((`xstop' - `xstart') / `xby')
	quietly count
	if r(N) < `obsrequired' {
		quietly set obs `obsrequired'
		}

	******************************************************************************************************************
	*** initialize variables 
	******************************************************************************************************************

	* these are variables that store the threshold and net benefit
	tempvar threshold all none
	quietly g `threshold' = .
	quietly g `all' = .
	quietly g `none' = 0
	label var `threshold' "Threshold probability"
	label var `all' "all"
	label var `none' "none"


	* these are variables used to get the kaplan-meier estimates
	tempvar uni function ind s2 atrisk
	quietly g `uni' = uniform()
	
	
	******************************************************************************************************************
	*** fit models, save out predicted probabilities, and set up variables which will contain net benefits
	******************************************************************************************************************

	local index = 0
	local models = ""
	global modellist = ""

	foreach p of local predictors {
		local index = `index' + 1
		g p`index' = `p'
		local models = "`models'" + " p`index'"
		quietly g modelp`index' = .
		label var modelp`index' "`p'"
		global modellist = "$modellist" + "  " + "modelp`index'"
		}

	global numberofmodels=wordcount("$modellist")
	
	** set sample = 1 if data are non-missing for all models
	** (the goal is to compare the benefits for the same sample of patients)
	local temp = "p1 < ."
	if `index' > 1 {
		forvalues i=1(1)`index' {
			local temp = "`temp'" + " & p`i' < . "
			}
		}
	g sample = `temp'
	


	******************************************************************************************************************
	*** calculate net benefit for each threshold
	*** assume the data are already -stset-
	******************************************************************************************************************


	* get the probability of the event for all subjects
	* this is used for the net benefit associated with treating all patients

	* if cmprsk option is not specified, then use Kaplan-Meier estimates
	if "`cmprsk'"=="" {
		quietly sts gen `function' = s
		quietly sort _t
		quietly g `ind' = cond(_t <=`timepoint', 1, .)
		quietly g `s2' = `function' * `ind'
		quietly sum `s2'
		scalar pd = 1 - r(min)
		drop `function' `ind' `s2'
		}
	* else if cmprisk option is specified, use the cumulative incidence function
	else {
		quietly stcompet `function' = ci, compet1(2)
		quietly sort _t
		quietly g `ind' = cond(_t <=`timepoint', 1, .)
		quietly g `s2' = `function' * `ind' if `cmprsk'==1
		quietly sum `s2'
		scalar pd = r(max)
		drop `function' `ind' `s2'
		}

	local l = 1
	forvalues j=`xstart'(`xby')`xstop' {
		quietly replace `threshold'=`j' in `l'
		local allnb = pd - (1 - pd) * `j' / (1 - `j')
		quietly replace `all' = `allnb' in `l'
		foreach m of local models {
			* get the probability of the event for subjects above the threshold
			quietly g `atrisk' = `m' >= `j' & `m' < .
			quietly sum `atrisk'
			if r(mean) ~=0 {
				* if comprsk option is not specified, use kaplan-meier methods
				if "`cmprsk'"=="" {
					quietly sts gen `function' = s if `atrisk'==1
					gsort -`atrisk' _t
					quietly g `ind' = cond(_t <=`timepoint', 1, .)
					quietly g `s2' = `function' * `ind'
					quietly sum `s2' if `atrisk'==1
					scalar pdgivenx = 1 - r(min)
					quietly sum `atrisk'
					scalar px = r(mean)
					quietly count
					local a = pdgivenx * px * r(N)
					local b = (1 - pdgivenx) * px * r(N)
					local u=(`a'-`b'*(`j'/(1-`j')))/r(N)
					drop `function' `ind' `s2'
					}
				* else if cmprisk option is specified, use the cumulative incidence function	
				else {
					quietly stcompet `function' = ci if `atrisk'==1, compet1(2)
					gsort -`atrisk' _t
					quietly g `ind' = cond(_t <=`timepoint', 1, .)
					quietly g `s2' = `function' * `ind' if `cmprsk'==1
					quietly sum `s2' if `atrisk'==1
					scalar pdgivenx = r(max)
					quietly sum `atrisk'
					scalar px = r(mean)
					quietly count
					local a = pdgivenx * px * r(N)
					local b = (1 - pdgivenx) * px * r(N)
					local u=(`a'-`b'*(`j'/(1-`j')))/r(N)
					drop `function' `ind' `s2'
					}
				}
			else {
				local u = .
				}

			sort `uni'
			quietly replace model`m' =`u' - ``m'harm' in `l'
			drop `atrisk'
	
			}
		local l=`l'+1
		}

	*put threshold on a percentage scale
	quietly replace `threshold'=`threshold'*100
	
	* transform net benefit to interventions, if specified
	if "`intervention'" ~="" {
		foreach var of varlist $modellist {
			quietly replace `var'= (`var' - `all') * 100 / ( `threshold' / (100 - `threshold') )
			if "`intermin'" ~="" {
				quietly replace `var' = . if `var' < `intermin'
				}
			}	
		}


	else {
		* cut off the curves if they are too low
		foreach var of varlist $modellist `all' {
			quietly replace `var'=. if `var'< `ymin' | `var' > `ymax' 
			}
		}

	
	******************************************************************************************************************
	*** save output from decision curve analysis
	******************************************************************************************************************
	keep $modellist `all' `none' `threshold'
	quietly drop if `threshold'==.
	* the temp variables currently have anonomous names
	* rename them here
	rename __000000 threshold
	rename __000001 all
	rename  __000002 none
	if "`saving'" ~="" {
		save "`saving'", `replace'
		}

	******************************************************************************************************************
	*** apply smoothing, if specified
	******************************************************************************************************************
	if "`smooth'" ~= "" {
		foreach var of varlist $modellist {
			quietly smooth `smooth' `var', gen(sm`var')
			quietly drop `var'
			quietly rename sm`var' `var'
			}
		}
	
	******************************************************************************************************************
	*** graph
	******************************************************************************************************************
	if "`graph'"=="yes"{

		if "`intervention'" == "" {

			if `ymin' > 0 {
				twoway (line all $modellist threshold if threshold>=`xstart'*100 & threshold<=`xstop'*100, sort  /*
				*/ clpat(solid `clpat') clwidth(medthick medium medium medium) clcolor(gs0 gs10 `clcolor')), /*
				*/  scheme(s1mono) xtitle(Threshold probability in %, margin(medium)) ytitle(Net Benefit, margin(medium)) /*
				*/  ylabel(, format(%9.`decimal'f)) legend(`legend')
				}
		
			else {
				twoway (line none all $modellist threshold if threshold>=`xstart'*100 & threshold<=`xstop'*100, sort  /*
				*/ clpat(solid solid `clpat') clwidth(medthick medthick medium medium medium) clcolor(gs0 gs10 `clcolor')), /*
				*/  scheme(s1mono) xtitle(Threshold probability in %, margin(medium)) ytitle(Net Benefit, margin(medium)) /*
				*/  ylabel(, format(%9.`decimal'f)) legend(`legend')
				}
		
			*************************************************************************************************************
			*** print out a line of text for a figure title
			*************************************************************************************************************
			
			if "`legend'" == "off" {
				disp ""
				disp "Figure legend: "
				if `ymin' > 0 {
					disp "Solid black line is treat none"
					}
				disp "Solid grey line is treat all"
				forvalues i=1(1)$numberofmodels{
					disp proper(word("`clpat'",`i')), word("`clcolor'",`i') " line is treat based on model:",  word("`predictors'",`i')
					}
				}
			}

		else {
			twoway (line $modellist threshold if threshold>=`xstart'*100 & threshold<=`xstop'*100, sort  /*
			*/ clpat(`clpat') clwidth(medium medium medium) clcolor(`clcolor')), /*
			*/  scheme(s1mono) xtitle(Threshold probability in %, margin(medium)) ytitle("Net reduction in interventions" "per 100 patients", margin(medium)) /*
			*/  ylabel(, format(%9.0f)) legend(`legend')
		
		
			*************************************************************************************************************
			*** print out a line of text for a figure title
			*************************************************************************************************************
			
			if "`legend'" == "off" {
				disp ""
				disp "Figure legend: "
				forvalues i=1(1)$numberofmodels{
					disp proper(word("`clpat'",`i')), word("`clcolor'",`i') " line is treat based on model:",  word("`predictors'",`i')
					}
				}
			}


		}
	restore

end
exit
